/* Projector Color Sensor Calibration Routine
	written by Sam Bobb, August 2009 */

#include "color_cal.h"
#include "global_vars.h"
#include "e_led.h"
#include "wheel_speed_coordinator.h"
#include "e_motors_swarm.h"
#include "send_packet.h"



extern unsigned char T1_INT_FLAG;
	
void calibrate_color(void){
	int l_speed;		//left wheel speed
	int r_speed;		//right wheel speed

	float robotFrontX;  //Robot's position/orientation
	float robotFrontY;
	float robotTheta;
	
	unsigned char loop_counter = 0;

	float current_target[2] = {START_X, START_Y};

	unsigned char turn_flag = 0;
	unsigned char zig_zag_mode = 0;
	int direction = 1;


	while(zig_zag_mode < 4){
		if (T1_INT_FLAG == 1){	//this flag is set every 0.2 seconds
		
			e_set_led(1,2); // reverse led 0 so we know it's running
			
			
			e_get_configuration_front(&robotFrontX, &robotFrontY, &robotTheta);

			
			if ( (zig_zag_mode == 0) || (zig_zag_mode == 3) ) {
				// zig zag, slicing the y axis
				
				if ( ( fabs(robotFrontX - current_target[0]) < 20.0 ) && ( fabs(robotFrontY - current_target[1]) < 20.0 ) )
				{
					// check if we've reached our final point
					if ( (NUM_Y_DIVS % 2 == 0) && (current_target[0] == START_X + AREA_X) && (current_target[1] == START_Y + AREA_Y) ) 
					{
						zig_zag_mode++;
						turn_flag = 0;
						turn_flag = 0;
					} 
					else if ( ( NUM_Y_DIVS % 2 == 1 ) && (current_target[0] == START_X) && (current_target[1] == START_Y + AREA_Y)  )
					{
						zig_zag_mode++;	
					}	
					
					// check if we need to make a turn east or west
					else if ( turn_flag == 0 )
					{
						if ( current_target[0] == START_X ) 
						{ 
							current_target[0] += AREA_X; 
						}
						else
						{
							current_target[0] = START_X;
						}	
						turn_flag = 1;
					} 
					
					// we need to make a turn north
					else // turn flag == 1
					{
						current_target[1] += direction*INCREMENT_Y;
	
						turn_flag = 0;
					}
				}
				
				
				
			} 
			else if ( (zig_zag_mode == 1) || (zig_zag_mode == 2) ) 
			{
				// zig zag, slicing the x axis
				
				// are we at our destination?
				if ( ( fabs(robotFrontX - current_target[0]) < 20.0 ) && ( fabs(robotFrontY - current_target[1]) < 20.0 ) )
				{
					// check if we've reached our final point for this zig zag mode
					if ( (NUM_Y_DIVS % 2 == 0) && (NUM_X_DIVS % 2 == 0) && (current_target[0] == START_X) && (current_target[1] == START_Y) ) 
					{
						zig_zag_mode++;
						direction = -1;
						
						
					} 
					else if ( ( NUM_Y_DIVS % 2 == 0 ) && (NUM_X_DIVS % 2 == 1) && (current_target[0] == START_X) && (current_target[1] == START_Y + AREA_Y)  )
					{
						zig_zag_mode++;	
					}	
					else if ( ( NUM_Y_DIVS % 2 == 1 ) && (NUM_X_DIVS % 2 == 0) && (current_target[0] == START_X + AREA_X) && (current_target[1] == START_Y)  )
					{
						zig_zag_mode++;	
					}
					else if ( ( NUM_Y_DIVS % 2 == 1 ) && (NUM_X_DIVS % 2 == 1) && (current_target[0] == START_X + AREA_X) && (current_target[1] == START_Y + AREA_Y)  )
					{
						zig_zag_mode++;	
					}
					
					// check if we need to make a turn north or south
					else if ( turn_flag == 0 )
					{
						if ( current_target[1] == START_Y ) 
						{ 
							current_target[1] += AREA_Y; 
						}
						else
						{
							current_target[1] = START_Y;
						}	
						turn_flag = 1;
					} 
					
					// we need to make a turn east or west
					else // turn flag == 1
					{
						if ( (NUM_Y_DIVS % 2 == 0) ){ // even, move west
							current_target[0] -= direction*INCREMENT_X;
						}
						else // odd, move east
						{
							current_target[0] += direction*INCREMENT_X;
						}		
	
						turn_flag = 0;
					}
				}
				
				
			}	
				
				
			
			
			//if ( ( fabs(AREA_X - current_target[0]) < 100.0 ) && ( fabs(AREA_Y - current_target[1]) < 100.0 ) ) 						

			//wheelSpeedSingleBot(pointlist[current_point][0], pointlist[current_point][1], &l_speed, &r_speed);		//Calculate new wheel speeds to use
			
			
			wheelSpeedSingleBot(current_target[0], current_target[1], &l_speed, &r_speed);		//Calculate new wheel speeds to use
			
			e_set_speed_left(l_speed);			//set wheel speeds
			e_set_speed_right(r_speed);
			
			
						
			
			if (loop_counter >= 1){  //calculate estimate and send data every other loop to prevent packet flooding			
				send_packet();
				loop_counter = 0;
			}
			else{
				loop_counter++;
			}				


			T1_INT_FLAG = 0;
		}


	}





}



float fabs(float input)
	{
	if (input<0)
		{
		input= -input;
		}
	return input;
	}

