#include "p30f6014A.h"
#include "e_init_port.h"
#include "e_epuck_ports.h"
#include "e_led.h"
#include "e_motors_swarm.h"
#include "e_acc.h"
#include "e_ad_conv.h"

#include "global_vars.h"

#include "main.h"
#include "dsPIC_XBeePackets.h"
#include "PI_consensus_estimator.h"
#include "wheel_speed_coordinator.h"
#include "color_cal.h"

#include <uart.h>

#include <stdlib.h>

//edited out line 212, and the color_cal.c file

//variables
int __C30_UART=2;
extern unsigned char T1_INT_FLAG;
unsigned char loop_counter = 0;


extern volatile float x_i[NUM_DATA_SETS];	//x, current_estimate/decision variables
extern volatile float u_i[NUM_DATA_SETS];	//u, Controller inputs
extern volatile float w_i[NUM_DATA_SETS];	//w, internal estimator state
extern volatile float x_sum[NUM_DATA_SETS];	//running sums
extern volatile float w_sum[NUM_DATA_SETS];


extern float MAX_WHEEL_V_TICKS; //Max velocity; the larger this is, the less time you have
								 //to do other calculations.  Limit is probably around 750
extern float deadband;  //deadband value for low motor velocity
extern float COMMR;  //Max communication radius

extern float SAFEDIST;	//Distance in mm at which obstacle avoidance will start running.
extern float MINDIST;	//Distance in mm at which obstacle avoidance will dominate the robot's motion.

extern float u_x_ideal;
extern float u_y_ideal;



//Timer1 Interrupt; triggers 0.2s
void __attribute__((__interrupt__,auto_psv)) _T1Interrupt(void){
	IFS0bits.T1IF = 0; //clear interrupt flag
	T1_INT_FLAG = 1;	//Set the flag
}

	
//UART Receive Interrupt, triggers when data comes in on serial port
void __attribute__((__interrupt__,auto_psv)) _U2RXInterrupt(void){
	char successFlag ;
	//extern volatile struct IN_PACKET InPacket;
	

	extern volatile struct IN_PACKET InPacket;
	
	int sw_val;

	float dist1;
	float dist2;	

	while (!DataRdyUART2());
	successFlag=readPacket();
	notRTS=1;						//Assert flow control line to stop XBee from sending

	if (successFlag == 0){			//If no errors, add new data to x_sum
		e_set_led(7,2);             //successfully received the packet, toggle LED to indicate
		//Check to see who packet is from
		if (InPacket.source==0){	//If message came from base station...
			sw_val = (int)InPacket.data[0].dataFloat;	//The first byte indicates the function of the packet
			switch (sw_val){
				/*
				Command IDs:
				command		ID		Description      
				(n/a)		0.0		 coordinate data
				sleep		1.0		 sleep
				wake		2.0		 wake (resets x_i consensus values)
				goal		3.0		 change goal parameters
				goto		4.0		 change goal position only (formation is the same)
				deadb		5.0		 change	motor low-velocity deadband
				egain		6.0		 change estimator gains KP,KI,GAMMA
				cgain		7.0		 change controller gains
				commr		8.0		 change COMMR
				speed		9.0		 change MAX_WHEEL_V_TICKS
				safed		10.0	 change obstacle avoidance theshold SAFEDIST 
				*/
				int i = 0;
				case 0: //coordinate data
					e_set_configuration(
					InPacket.data[1].dataFloat,
					InPacket.data[2].dataFloat, 
					InPacket.data[3].dataFloat);
					break;
				case 1: //sleep
					IEC0bits.T1IE = 0;
					e_set_speed_left(0);
					e_set_speed_right(0);
					break;
				case 2: //wake up
					e_get_configuration(&robotX, &robotY, &robotTheta);
					x_i[0] = robotX;
					x_i[1] = robotY;
					x_i[2] = 0;
					x_i[3] = 0;
					x_i[4] = 0;
	
					srand(robotX); //seed random number generator
					delay((unsigned long)((float)rand()*30.0));	//random delay so that robots won't all try to send at the same time.
					
					for (i=0; i<NUM_DATA_SETS; i++){
						x_sum[i] = 0.0;
						w_sum[i] = 0.0;
						w_i[i] = 0.0;
					}
					
					IEC0bits.T1IE 	= 1;
					//Sets initial speed to Zero, should be removed before testing
					//{
					e_set_speed_left(0);
					e_set_speed_right(0);
					//}
					break;
				case 3:		//change goal
					goal[0] = InPacket.data[1].dataFloat;
					goal[1] = InPacket.data[2].dataFloat;
					goal[2] = InPacket.data[3].dataFloat;
					goal[3] = InPacket.data[4].dataFloat;
					goal[4] = InPacket.data[5].dataFloat;
					break;
				case 4:   //change goal, position only
					goal[0] = InPacket.data[1].dataFloat;
					goal[1] = InPacket.data[2].dataFloat;
					break;
				case 5: // change deadband
					deadband = InPacket.data[1].dataFloat;
					break;
				case 6: // change estimator gains
					KP = InPacket.data[1].dataFloat;
					KI = InPacket.data[2].dataFloat;
					GAMMA = InPacket.data[3].dataFloat;
					break;
				case 7: //change controller gains
					for (i = 0; i<5; i++){
						k[i] = InPacket.data[i+1].dataFloat;
					}
					break;	
				case 8:  //change COMMR
					COMMR = InPacket.data[1].dataFloat;
					break;
				case 9:  //change MAX_WHEEL_V_TICKS
					MAX_WHEEL_V_TICKS = InPacket.data[1].dataFloat;
					break;	
				case 10:   ///change obstacle avoidance theshold
					SAFEDIST = InPacket.data[1].dataFloat;
					break;				
				default:
					break;
			}	
		}
		else{	//If message did NOT come from base station...
			//get the nearest agent and its positions
			e_get_configuration(&robotX,&robotY,&robotTheta);
			
			//squared distance of closest known neighbor
			dist1 = SQUARE(nx-robotX)+SQUARE(ny-robotY);	
			
			//squared distance of current neighbor that is sending you data
			dist2 = SQUARE(InPacket.data[10].dataFloat-robotX)+SQUARE(InPacket.data[11].dataFloat-robotY);

			if (dist2<COMMR*COMMR){		//if this neighbor is within the communication radius
				//if this neighbor is closer than current known nearest neighbor, update data
				if ((nflag==0)||(nid == InPacket.source)||(dist2<dist1)){	
					nx = InPacket.data[10].dataFloat;
					ny = InPacket.data[11].dataFloat;
					nid = InPacket.source;
					nflag =1; 
				}
				if (getMsgRxFlag(InPacket.source) ==0){	//make sure it hasn't already sent you data
					addConsensusData();					//add the data to the consensus estimator
					setMsgRxFlag(InPacket.source,1);	//Set flag to indicate a message was received from this agent
				}
			}
		}
	}
	else if (successFlag == 2){		// command response success
		//e_set_led(5,2);
	}
	else if (successFlag <0){
		e_set_led(-successFlag,2);	// messege error
		U2STAbits.OERR=0;			//rearm the overflow error flag in case it was set
	}
	else {

	}
	IFS1bits.U2RXIF = 0;	//clear interrupt flag
	notRTS=0;	//De-assert flow control line, XBee can start sending again.
}

int main(void){
//	unsigned char AGENT_ID;
	float packet[DATA_ARRAY_LENGTH+ADDITIONAL_NUMS]={0};	//make array to hold data to send to radio
	
	// set up some variables to hold the RGB values
	float red_value, green_value, blue_value;	

	e_init_port();	//initialize ports


	// set up the accelerometer ADCs
	e_init_ad_scan(ALL_ADC);
	//e_acc_calibr(); // we're not going to calibrate, we'll leave the offset at 0


	
	//set notRTS pin as output (notRTS is defined as SELECTOR3)
	SELECTOR3_DIR = OUTPUT_PIN;
	SELECTOR2_DIR = INPUT_PIN;
	
	//wheels speeds set to 0 at start
	e_set_speed_left(0);
	e_set_speed_right(0);
	e_set_configuration(START_COORD_X, START_COORD_Y, START_THETA);

	//init USART 2
	U2MODE = 0b1000000000000000;
	U2STA  = 0b0000010000000000;
	U2BRG = ((FCY/115200)/16)-1;    //set baud rate
	IEC1bits.U2RXIE = 1;			//enable enterrupt
	IPC6  |= 0b0000000000000111;	//set high priority
	
//	AGENT_ID = getMY_ID;

	//init Timer1
	T1CONbits.TCKPS = 0b11;
	T1CONbits.TON	= 1;
	PR1 = 11519;    //this timer period will make Timer1 trigger an interrupt ever 0.2 seconds
	IEC0bits.T1IE 	= 0;  //Go to sleep when start. This just disables timer1.


	
	// A call to the calibrate_color function for testing.
	// Eventually you'll want to add a command to the Xbee interrupt for "Calibrate Color" and also add to
	// the vision system commands. Then the only action from the "Calibrate Color" command will be a call to
	// calibrate_color()
	calibrate_color();
	
	
	
	while(1){		
		if (T1_INT_FLAG == 1){	//this flag is set every 0.2 seconds
			unsigned int i=0;	//counter variable
			unsigned int j=0;	//another counter variable
			int l_speed;		//left wheel speed
			int r_speed;		//right wheel speed

			e_set_led(0,2);                   // reverse led status so we know it's running
			wheelSpeed(&l_speed, &r_speed);		//Calculate new wheel speeds to use
			e_set_speed_left(l_speed);			//set wheel speeds
			e_set_speed_right(r_speed);
			
			if (loop_counter >= 1){  //calculate estimate and send data every other loop to prevent packet flooding			
				calcNewEstimate();  	//Calculate new estimate
				resetMsgRxFlags();	//Reset all message flags
			
				for (i=0,j=0;i<DATA_ARRAY_LENGTH;){
					packet[i]=x_i[j];	//packet is structured x_1, w_1, x_2, w_2, etc.
					i++;
					packet[i]=w_i[j];
					i++;
					j++;
				}	
				//add on additional numbers to packet.  These numbers are used for data logging, not consensus estimation.
				e_get_configuration(&robotX, &robotY, &robotTheta);
				packet[i]=robotX;
				i++;
				packet[i]=robotY;
				i++;
				packet[i]=robotTheta;
				i++;
				packet[i]=u_x_ideal;
				i++;
				packet[i]=u_y_ideal;	

				// Grab values from the color sensor. Average the last 136 samples.
				// The projector has a PWM period of about 8ms.
				// The puck samples the color sensor at 4096 Hz (set in e_ad_conv.h).
				// 136 samples gives us an average over about 4 cycles of the projection
				// which seems to give a pretty decent reading from the projector.
				blue_value=e_get_acc_filtered(0, 136);
            	green_value=e_get_acc_filtered(1, 136);
            	red_value=e_get_acc_filtered(2, 136);

				i++;
				packet[i]=red_value;
				i++;
				packet[i]=green_value;
				i++;
				packet[i]=blue_value;
				
				notRTS=1;	//assert flow control line to stop XBee from sending
				assemblePacket(packet, 0xFFFF);	//assemble and send packet
				notRTS=0;	//deassert flow control; Xbee can send again
				
				loop_counter = 0;
			}
			else{
				loop_counter++;
			}				
			T1_INT_FLAG = 0;
		}
	}
	
	return 0;
}
